/* decode.c - The EuroCrypt decryption algorithm.

   (c) Copyright 1996, Marcus Better.

   Last updated: 1996-04-04
*/

#include "des.h"
#include "decode.h"

#define DES_ITER 16

unsigned long F(unsigned long, unsigned char *);

void decode(unsigned char *in, const unsigned char *key)
{
  int i;
  unsigned long R, L, C, D;

  C = (unsigned long)key[0] << 20
    ^ (unsigned long)key[1] << 12
    ^ (unsigned long)key[2] <<  4
    ^ (unsigned long)(key[3]>>4);
  D = (unsigned long)(key[3]&0x0f) << 24
    ^ (unsigned long)key[4] << 16
    ^ (unsigned long)key[5] << 8
    ^ (unsigned long)key[6];

  L = (unsigned long)in[0] << 24
    ^ (unsigned long)in[1] << 16
    ^ (unsigned long)in[2] << 8
    ^ (unsigned long)in[3];
  R = (unsigned long)in[4] << 24
    ^ (unsigned long)in[5] << 16
    ^ (unsigned long)in[6] << 8
    ^ (unsigned long)in[7];

  for(i=0; i<DES_ITER; i++) {
    unsigned long R1;
    unsigned char K[8];
    int j, k;

    /* Key schedule */
    for( j=0; j<LS[i]; j++ ) {
      C = (C<<1 ^ C>>27) & 0xfffffffL;
      D = (D<<1 ^ D>>27) & 0xfffffffL;
    }
    for( j=0, k=0; j<8; j++ ) {
      int t;
      K[j]=0;
      for(t=0; t<6; t++, k++)
        if( PC2[k] < 29 )
          K[j] |= (C >> 28-PC2[k] & 1) << (5-t);
        else
          K[j] |= (D >> 56-PC2[k] & 1) << (5-t);
    }

    /* One decryption round */
    R1 = L ^ F(R, K);
    L = R;
    R = R1;
  }

  in[0] = R>>24;
  in[1] = R>>16;
  in[2] = R>>8;
  in[3] = R;
  in[4] = L>>24;
  in[5] = L>>16;
  in[6] = L>>8;
  in[7] = L;
}

unsigned long F(unsigned long R, unsigned char *K)
{
  int i, k;
  unsigned long s=0, result=0;

  for(i=0, k=0; i<8; i++) {
    int j;
    char v=0;
    /* The expansion E */
    for(j=0; j<6; j++, k++)
      v |= (R>>32-E[k] & 1) << (5-j);
    v ^= K[i];
    /* The S-boxes */
    s |= (unsigned long)S[i][v] << 28-4*i;
  }

  /* The permutation P */
  for(i=0; i<32; i++)
    result |= (s>>32-P[i] & 1) << 31-i;

  return result;
}