/* eurocry.c - EuroCrypt smart-card emulator.

   (c) Copyright 1996, Marcus Better.

   Last updated: 1996-04-04
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>

#include "eurocry.h"
#include "decode.h"
#include "async.h"

/* Edit these to match your configuration */
#define PORT COM2

#define BAUD_RATE 10000
/*
#define BAUD_RATE 9600
*/

int display=1;

main()
{
  unsigned char header[5], data[64], ident[3], des_data1[8], des_data2[8];
  int quit=0, count=0, data_len, active_channel=-1;
  unsigned char id_88, len_88;
  enum states state;
  enum states_88 state_88;

  activate_com();
  AsyncStat();   /* clear D_DCD */

  textcolor(LIGHTGRAY);
  cprintf("Waiting for reset...\n\r");
  while( !quit ) {
    if( kbhit() ) {
      quit=1;
      continue;
    }
    if (AsyncStat() & (DCD | D_DCD)) {
      textcolor(LIGHTGRAY);
      cprintf("RESET ");
      if( AsyncStat() & DCD )
        AsyncClear();
      while ((AsyncStat() & DCD) && !kbhit());
      if( kbhit() ) continue;

      wait_us(delus[0]);
      if (send(reset_answer, reset_answer_len)) continue;
      state=REC_HEADER;
      count=0;
      AsyncStat(); /* Clear D_DCD */
    }
    if( state==REC_HEADER ) {
      if( count < HEADER_LEN )
        count += receive(header+count, HEADER_LEN-count);
      else {
        display_header(header);
        switch( header[1] ) {
          case 0x02:
          case 0xa4:
            if( header[4] ) {
              send_byte(header[1]);
              count=0;
              state=REC_DATA;
              data_len=header[4];
            }
            else {
              send_eol();
              state=REC_HEADER;
              count=0;
            }
            break;
          case 0x04:
            send_byte(header[1]);
            send(answer_04, answer_04_len);
            state=REC_HEADER;
            count=0;
            break;
          case 0x06:
            send_byte(header[1]);
            send(answer_06, answer_06_len);
            state=REC_HEADER;
            count=0;
            break;
          case 0x24:
            send_rep(0, header[4]);
            send_eol();
            state=REC_HEADER;
            count=0;
            break;
          case 0x26:
            send_byte(0x6e);
            send_byte(0x00);
            state=REC_HEADER;
            count=0;
            break;
          case 0x88:
            send_byte(0x77);
            data_len=header[4]-8;
            state=REC_DESDATA;
            state_88=REC_88_ID;
            count=0;
            break;
          case 0xac:
            send_eol();
            state=REC_HEADER;
            count=0;
            break;
          case 0xb8:
            send_byte(header[1]);
            send_rep(0, header[4]);
            send_err();
            state=REC_HEADER;
            count=0;
            break;
          case 0xc0:
            send_byte(header[1]);
            send_byte(0xea);
            send_byte(0x10);
            send(des_data1, 8);
            send(des_data2, 8);
            send_eol();
            state=REC_HEADER;
            count=0;
            break;
          default:
            cprintf("Unknown command!");
            send_err();
            state=REC_HEADER;
            count=0;
        }
      }
    }
    else if( state==REC_DATA ) {
      if( count<data_len )
        count += receive(data+count, data_len-count);
      else {
        display_data(data, data_len);
        switch( header[1] ) {
          int i;
          case 0x02:
            for( i=0; i<data_len; i++ )
              if( data[i]==0x40 ) {
                send_err();
                break;
              }
            if( i==data_len )
              send_eol();
            state=REC_HEADER;
            count=0;
            break;
          case 0xa4:
            send_eol();
            memcpy(ident, data, 3);
            state=REC_HEADER;
            count=0;
            break;
          case 0x88:
            active_channel=-1;
            textcolor(LIGHTGRAY);
            for( i=0; i<CHANNELS; i++ )
              if( !memcmp(channel[i].ident, ident, IDENT_LEN) &&
                   header[3]==channel[i].index ) {
                active_channel=i;
                cprintf("\n\rChannel: %s", channel[i].name);
                break;
              }
            if( i==CHANNELS )
              cprintf("\n\rChannel unknown!");
            if( active_channel>=0 ) {
              decode(des_data1, channel[active_channel].key);
              decode(des_data2, channel[active_channel].key);
            }
            else {
              memset(des_data1, 0, 8);
              memset(des_data2, 0, 8);
            }
            send_eol();
            state=REC_HEADER;
            count=0;
            break;
          default:
            cprintf("\n\rUnhandled data!");
        }
      }
    }
    else /* if( state==REC_DESDATA ) */ {
      if( state_88 == REC_88_ID ) {
        if( receive(&id_88, 1) ) {
          state_88=REC_88_LEN;
          send_byte(0x77);
        }
      }
      else if( state_88 == REC_88_LEN ) {
        if( receive(&len_88, 1) ) {
          count=0;
          if( id_88==0xf0 ) {
            state=REC_DATA;
            data_len = len_88;
            send_byte(0x88);
          }
          else {
            state_88=REC_88_DATA;
            send_byte(0x77);
          }
        }
      }
      else /* if( state_88==REC_88_DATA ) */ {
        if( count<len_88 ) {
          if( receive(data+count, 1) ) {
            count++;
            send_byte(0x77);
          }
        }
        else {
          display_data(&id_88, 1);
          display_data(&len_88, 1);
          display_data(data, len_88);
          if( id_88==0xea ) {
            memcpy(des_data1, data, 8);
            memcpy(des_data2, data+8, 8);
          }
          state_88=REC_88_ID;
        }
      }
    }
  }
  getch();

  deactivate_com();
  return 0;
}

/*
 * Send a string of bytes to serial port and wait for each single echo.
 * (More efficient send methods are too fast for decoder, it seems to
 * need more than 2 stop bits, so we wait between individual bytes.)
 */
int send(const unsigned char *data, int len)
{
  int i;
  char c;

  AsyncClear();
  for (i = 0; i < len; i++) {
    c = data[i];
    inverse(&c, 1);
    AsyncOut(c);
    while (AsyncInStat() == 0)
      if ((AsyncStat() & DCD) || kbhit()) return 1;
    AsyncIn();
    wait_us(delus[1]);
  }

  if(display)
    display_out(data, len);

  return 0;
}

/*
 * Get all bytes available from serial port FIFO and return how many
 * bytes were available.
 */
int receive(unsigned char *data, int max)
{
  int i = 0;

  while (AsyncInStat() > 0 && i < max)
    data[i++] = AsyncIn();
  inverse(data, i);

  return i;
}

/*
 * Reverse and invert all bits in each byte of a string.
 * E.g. 10010111b (0x97) becomes 00010110b (0x16).
 */
void inverse(char *data, int len)
{
  int i, j;
  unsigned char c;

  for (i = 0; i < len; i++) {
    c = 0;
    for (j = 0; j < 8; j++)
      c |= ((data[i] & (1 << j)) != 0) << (7 - j);
    data[i] = ~c;
  }

  return;
}

/* Uses a BIOS function in order to wait a specified number of
 * microseconds. This BIOS functions is only implemented in
 * IBM AT compatible BIOSes (i.e., not in old PC and XT systems)!
 * The timing resolution of this function isn't really down
 * to 1 us on most systems, but it's fine for our purposes.
 */
void wait_us(unsigned long microseconds)
{
  union REGS regs;

  if (microseconds == 0) return;
  regs.h.ah = 0x86;
  regs.x.cx = (int) (microseconds >> 16);
  regs.x.dx = (int) (microseconds & 0xffff);
  int86(0x15, &regs, &regs);
  return;
}

/* Setup the serial port */
void activate_com(void)
{
  if (AsyncInit(PORT)) {
    printf("Can't initialize port!\n");
    exit(1);
  }
  AsyncHand(DTR | RTS);
  AsyncSet(BAUD_RATE, BITS_8 | STOP_2 | ODD_PARITY);

  return;
}

/* Stop the serial communication */
void deactivate_com(void)
{
  AsyncStop();
}

/* Send normal status reply (90h, 00h) to the decoder */
void send_eol(void)
{
  send_byte(0x90);
  send_byte(0x00);
}

/* Send error reply (90h, 08h) to the decoder */
void send_err(void)
{
  send_byte(0x90);
  send_byte(0x08);
}

/* Send a byte repeatedly */
void send_rep(unsigned char c, int n)
{
  int i;
  for( i=0; i<n; i++ )
    send(&c, 1);
}

/* Send one byte without printing it on the screen */
void send_byte(unsigned char c)
{
  display=0;
  send(&c, 1);
  display=1;
}

/* Print output bytes as hex */
void display_out(const unsigned char *data, int len)
{
  int i;

  textcolor(LIGHTGRAY);
  for( i=0; i<len; i++ )
    cprintf("%02X ", (unsigned)data[i]&0xff);
}

/* Print received header bytes as hex */
void display_header(const unsigned char *header)
{
  int i;

  textcolor(WHITE);
  cprintf("\n\r");
  for( i=0; i<HEADER_LEN; i++ )
    cprintf("%02X ", (unsigned)header[i]&0xff);
  textcolor(LIGHTGRAY);
  cprintf(": ");
}

/* Print received data bytes as hex */
void display_data(const unsigned char *data, int len)
{
  int i;

  textcolor(LIGHTRED);
  for( i=0; i<len; i++ )
    cprintf("%02X ", (unsigned)data[i]&0xff);
}
