#ifndef _EUROCRY_H
#define _EUROCRY_H

/* Data lengths etc */
#define HEADER_LEN        5
#define IDENT_LEN         3

/* The channels */
/* These structures describe the channels, with identifiers and keys.
   Note that there are no actual keys in this example.
*/
#define CHANNELS         13
const struct {
  char name[40];
  char ident[IDENT_LEN];
  char index;
  unsigned char key[7];
} channel[CHANNELS] = {
  { "TV3",
    {0x00, 0x04, 0x00},
    0x08,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "TV3",
    {0x00, 0x04, 0x00},
    0x0a,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "TV3",
    {0x00, 0x04, 0x00},
    0x0b,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "TV1000",
    {0x00, 0x04, 0x10},
    0x0b,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "FilmNet",
    {0x00, 0x28, 0x00},
    0x0a,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "CTV",
    {0x00, 0x04, 0x30},
    0x08,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "BBC Prime",
    {0x00, 0x28, 0x10},
    0x08,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "TV Plus",
    {0x00, 0x2c, 0x00},
    0x08,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "Canal+",
    {0x10, 0x00, 0x20},
    0x05,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "Canal+",
    {0x10, 0x00, 0x20},
    0x06,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "Cine Cinemas",
    {0x10, 0x00, 0x30},
    0x05,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "Cine Cinemas",
    {0x10, 0x00, 0x30},
    0x06,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  },
  { "Cine Cinephile",
    {0x00, 0x24, 0x20},
    0x08,
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}
  }
};

/* The states */
enum states{
  REC_HEADER, REC_DATA, REC_DESDATA
};

enum states_88 {
  REC_88_ID, REC_88_LEN, REC_88_DATA
};

/* Delays in microseconds */
#define DELAYS 2
unsigned long delus[DELAYS] = {
  30000,      /* wait between reset and answer to reset */
  500,        /* wait after each outgoing byte */
};

/* Hard-coded replies */
#define reset_answer_len 11
const unsigned char reset_answer[reset_answer_len] =
    { 0x3f, 0x67, 0x2f, 0x00,
      0x11, 0x14, 0x00, 0x03,
      0x68, 0x90, 0x00 };

#define answer_04_len 9
const unsigned char answer_04[32] =
    { 0x00, 0x15, 0x00, 0x04,
      0x00, 0x00, 0x00, 0x90,
      0x00 };

#define answer_06_len 6
const unsigned char answer_06[32] =
    { 0x10, 0x02, 0xca, 0x20,
      0x90, 0x00 };

/* Functions declarations */
int send(const unsigned char *, int);
int receive(unsigned char *, int);
void inverse(char *, int);
void wait_us(unsigned long);
void activate_com(void);
void deactivate_com(void);
void display_out(const unsigned char *, int);
void display_header(const unsigned char *);
void display_data(const unsigned char *, int);
void send_eol(void);
void send_err(void);
void send_rep(unsigned char, int);
void send_byte(unsigned char);

#endif  /* _EUROCRY_H */